package cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex;

import cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap.Heatmap;
import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4HunterService;
import cz.mendelu.dnaAnalyser.exporter.ExporterService;
import cz.mendelu.dnaAnalyser.sequence.SequenceService;
import cz.mendelu.dnaAnalyser.utils.controller.*;
import cz.mendelu.dnaAnalyser.utils.controller.GetMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import cz.mendelu.dnaAnalyser.utils.analyse.data.ExporterEnums.Analysis;
import cz.mendelu.dnaAnalyser.utils.analyse.data.ExporterEnums.Output;

import java.util.Locale;
import java.util.UUID;

@RestController
@Api(tags = "api/analyse/g4hunter/{id}/quadruplex")
@RequestMapping("api/analyse/g4hunter/{id}")
public class QuadruplexController {


    @Autowired
    private G4HunterService g4HunterService;

    @Autowired
    private ExporterService exporterService;

    @Autowired
    private SequenceService sequenceService;

    @GetMapping("quadruplex")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Get one G4hunter analyse by ID.")
    public ResponsePage<AggregateQuadruplex> getQuadruplexList(
            @PathVariable UUID id,
            Integer sequenceStart,
            Integer sequenceLength,
            PaginationParam paginationParam,
            SortParam sortParam)
    {
        Pageable pageable = paginationParam.toPageable(sortParam.toSort());
        Page<AggregateQuadruplex> quadruplexPage = g4HunterService.analyseQuadruplex(id, sequenceStart, sequenceLength, pageable);
        return ResponsePage.responsePage(quadruplexPage);
    }

    @GetMapping(
            path = "quadruplex.csv",
            produces = MediaType.TEXT_PLAIN_VALUE
        )
    @ResponseStatus(HttpStatus.OK)
    //@PreAuthorize("isAuthenticated()")
    @ApiOperation("Get CSV Export.")
    public ResponseEntity<String> getCSV(
            @PathVariable UUID id,
            boolean aggregate)
    {
        String csv = aggregate
                ? exporterService.exportData(id, Analysis.AGGREGATE, Output.CSV)
                : exporterService.exportData(id, Analysis.QUADRUPLEX, Output.CSV);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentDisposition(ContentDisposition
                        .builder("attachment")
                        .filename(id + ".csv")
                        .build());
        headers.setContentType(MediaType.valueOf("text/csv"));
        headers.setContentLength(csv.length());

        return new ResponseEntity<>(csv, headers, HttpStatus.OK);
    }

    @GetMapping(
        path = "quadruplex.bedgraph",
        produces = MediaType.TEXT_PLAIN_VALUE
    )
    @ResponseStatus(HttpStatus.OK)
    @ApiOperation("Get BEDGRAPH Export.")
    public ResponseEntity<String> getBED(
            @PathVariable UUID id)
    {
        String bedGraph = exporterService.exportData(id, Analysis.QUADRUPLEX, Output.BEDGRAPH);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentDisposition(ContentDisposition
                        .builder("attachment")
                        .filename(id + ".bedgraph")
                        .build());
        headers.setContentType(MediaType.valueOf("text/bedgraph"));
        headers.setContentLength(bedGraph.length());

        return new ResponseEntity<>(bedGraph, headers, HttpStatus.OK);
    }

    @GetMapping("/heatmap")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Get one G4hunter analyse by ID.")
    public ResponseEntity<Heatmap> getQuadruplexList(
            @PathVariable UUID id,
            int segments,
            int from,
            int to)
    {
        UUID sequenceId = g4HunterService.findOne(id).getSequenceId();
        int sequenceLength = sequenceService.findOne(sequenceId).getLength();
        
        if (from < 0 || to > sequenceLength) {
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }

        Heatmap heatmap = g4HunterService.getHeatmap(id, segments, from, to);
        return new ResponseEntity<>(heatmap, HttpStatus.OK);

    }
}
