package cz.mendelu.dnaAnalyser.utils.swagger;

import com.google.common.collect.Lists;
import cz.mendelu.dnaAnalyser.Application;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiKey;
import springfox.documentation.service.AuthorizationScope;
import springfox.documentation.service.Contact;
import springfox.documentation.service.SecurityReference;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.Docket;

import java.util.List;
import java.util.Objects;

@Configuration
public class SwaggerConfiguration {

    @Value("${swagger2.info.title}")
    private String title;

    @Value("${swagger2.info.description}")
    private String description;

    @Value("${swagger2.info.license}")
    private String license;

    @Value("${swagger2.info.licenseUrl}")
    private String licenseUrl;

    @Value("${swagger2.info.termsOfServiceUrl}")
    private String termsOfServiceUrl;

    @Value("${swagger2.info.version}")
    private String version;

    @Value("${swagger2.info.contact.name}")
    private String name;

    @Value("${swagger2.info.contact.url}")
    private String url;

    @Value("${swagger2.info.contact.email}")
    private String email;

    @Value("${security.jwt.header}")
    private String tokenHeader;

    @Bean
    public Docket docket() {
        return new Docket(DocumentationType.SWAGGER_2)
                //.groupName(groupName)
                .select()
                .apis(RequestHandlerSelectors.basePackage(Application.class.getPackage().getName()))
                .build()
                .apiInfo(new ApiInfoBuilder()
                        .title(title)
                        .description(description)
                        .license(license)
                        .licenseUrl(licenseUrl)
                        .termsOfServiceUrl(termsOfServiceUrl)
                        .version(version)
                        .contact(new Contact(name, url, email))
                        .build())
                .useDefaultResponseMessages(false)
                .securityContexts(Lists.newArrayList(securityContext()))
                .securitySchemes(Lists.newArrayList(apiKey()));
    }

    private ApiKey apiKey() {
        return new ApiKey("JWT token", tokenHeader, "header");
    }

    private SecurityContext securityContext() {
        return SecurityContext.builder()
                .securityReferences(defaultAuth())
                .forPaths(path ->
                    !Objects.equals("/api/jwt", path)
                )
                .build();
    }

    private List<SecurityReference> defaultAuth() {
        AuthorizationScope authorizationScope = new AuthorizationScope("global", "accessEverything");
        AuthorizationScope[] authorizationScopes = new AuthorizationScope[] {authorizationScope};
        return Lists.newArrayList(new SecurityReference("JWT token", authorizationScopes));
    }

}
