package cz.mendelu.dnaAnalyser.overlay.service

import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4Hunter
import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4HunterRepository
import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4HunterService
import cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex.AggregateQuadruplexMapper
import cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex.QuadruplexDataSessionService
import cz.mendelu.dnaAnalyser.overlay.AnalyseResult
import cz.mendelu.dnaAnalyser.overlay.LocalisedOverlay
import cz.mendelu.dnaAnalyser.overlay.OverlayController
import cz.mendelu.dnaAnalyser.overlay.OverlayItem
import cz.mendelu.dnaAnalyser.overlay.OverlayService
import cz.mendelu.dnaAnalyser.sequence.NCBISequenceService
import cz.mendelu.dnaAnalyser.sequence.Sequence
import cz.mendelu.dnaAnalyser.sequence.annotation.Annotation
import cz.mendelu.dnaAnalyser.sequence.annotation.AnnotationRepository
import cz.mendelu.dnaAnalyser.sequence.annotation.AnnotationService
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.Feature
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.FeatureDataSessionService
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData
import cz.mendelu.dnaAnalyser.utils.analyse.data.TestData
import cz.mendelu.dnaAnalyser.utils.analyse.data.TestDataMapper
import cz.mendelu.dnaAnalyser.utils.analyse.model.Analyse
import cz.mendelu.dnaAnalyser.utils.analyse.model.TestAnalyse
import cz.mendelu.dnaAnalyser.utils.analyse.repository.AnalyseRepository
import cz.mendelu.dnaAnalyser.utils.analyse.service.TestDataSessionService
import cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap.HeatmapService
import cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap.TestHeatmapService
import cz.mendelu.dnaAnalyser.utils.overlay.TestLocalisedOverlay
import org.apache.ibatis.session.RowBounds
import org.apache.ibatis.session.SqlSession
import spock.lang.Ignore
import spock.lang.Specification
import spock.lang.Unroll

import java.util.stream.Stream

class OverlayServiceTest extends Specification {

    Analyse analyse
    TestDataMapper dataMapper
    LocalisedOverlay localisedOverlay;
    AnalyseRepository analyseRepository;
    byte[] buffer
    String tmpDir
    QuadruplexDataSessionService quadruplexDataSessionService
    TestDataSessionService dataSessionService
    G4HunterService g4HunterAnalyseService
    OverlayService overlayService
    G4HunterRepository g4HunterAnalyseRepository;

    def setup() {
        analyse = new TestAnalyse(id: UUID.randomUUID(), finished: new Date())
        dataMapper = Mock(TestDataMapper)
        localisedOverlay = new TestLocalisedOverlay(TestDataMapper.class, dataMapper)

        //annotationService.ncbiSequenceService = Mock(NCBISequenceService)
        //overlayService.analyseRepository = analyseRepository = Mock(AnalyseRepository)
        g4HunterAnalyseRepository = Mock(G4HunterRepository)

        buffer = '103614\t107842\tmobile_element'.bytes
        tmpDir = File.createTempDir().getPath()
        quadruplexDataSessionService = new QuadruplexDataSessionService(
                datasourceUrl: "jdbc:h2:file:$tmpDir/analyse-id;DB_CLOSE_ON_EXIT=FALSE",
                datasourceUsername: '',
                datasourcePassword: '',
                datasourceDriverClassName: 'org.h2.Driver'
        )
        dataSessionService = new TestDataSessionService(
                datasourceUrl: "jdbc:h2:file:$tmpDir/analyse-id;DB_CLOSE_ON_EXIT=FALSE",
                datasourceUsername: '',
                datasourcePassword: '',
                datasourceDriverClassName: 'org.h2.Driver'
        )

        //overlayService.quadruplexDataSessionService = quadruplexDataSessionService

        /*def analyseResult = new AnalyseResult(
                position: 10000,
                length: 100
        )

        def overlayItem = new OverlayItem(
                //occureneces: new HashMap<String, Object>(),
                averageCoverage: 10000,
                averageLength: 100,
                totalLength: 1000
        )*/
    }

    @Ignore
    def "CreateOverlay"() {
        setup:

        def g4 = new G4Hunter()

        def feature = new Feature()
        feature.setStartPosition("103600")
        feature.setEndPosition("107900")
        feature.setFeatureType("mobile_element")

        SqlSession sqlSession = quadruplexDataSessionService.openDataSession(g4)

        g4HunterAnalyseRepository.save(g4);

        g4HunterAnalyseRepository.findById(g4.id)
        Map<String, OverlayItem> result = new HashMap<>()

        String labelGroupBy = "featureType"

        List<String> intervals = Arrays.asList("0","5")
        List<String> groupIntervals = Arrays.asList("0-5")

        boolean aggregated = false

        int from = 103000
        int to = 108000

        when:
        def res = overlayService.createOverlay(analyse, feature, sqlSession, result, labelGroupBy, intervals, groupIntervals, aggregated, from, to)
        then:
        res[0].get("positions")[0] == "103614"
        res[0].get("positions")[1] == "107842"
        res[0].get("type") == "mobile_element"
    }



    @Ignore
    @Unroll
    def "Simple overlay test"() {
        setup:
        analyse.sequence = new Sequence(length: length)
        dataMapper.getAllBetweenPositionOrderAsc(from, to, "POSITION", new RowBounds()) >> testData

        def feature = new Feature()
        feature.setStartPosition("103600")
        feature.setEndPosition("107900")
        feature.setFeatureType("mobile_element")

        def g4 = new G4Hunter()
        SqlSession sqlSession = dataSessionService.openDataSession(analyse)

        Map<String, OverlayItem> result = new HashMap<>()

        String groupBy = "featureType"
        String labelGroupBy = "featureType"

        List<Double>  intervals = null;
        List<String> groupIntervals = Arrays.asList("0-5")


        int f = 103000
        int t = 108000


        when:
        def res = localisedOverlay.annotationOverlay(TestDataMapper.class, feature, sqlSession, result, groupBy, labelGroupBy, intervals, groupIntervals, f, t)
//(TestDataMapper.class, feature, sqlSession, result, labelGroupBy, intervals, groupIntervals, f, t)
        then:
        res[0].get("positions")[0] == "103614"

        where:
        from | to | seg | length | counts          | coverages                 | testData
        0    | 10 | 5   | 10     | [0, 1, 0, 0, 0] | [0.0, 1.0, 1.0, 0.5, 0.0] | [new TestData(position: 2, length: 5)]
        0    | 10 | 5   | 10     | [0, 1, 1, 0, 0] | [0.0, 1.0, 1.0, 0.5, 0.0] | [new TestData(position: 2, length: 5), new TestData(position: 4, length: 3)]
    }
}
