package cz.mendelu.dnaAnalyser.sequence


import cz.mendelu.dnaAnalyser.sequence.data.SequenceData
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide
import spock.lang.Specification

class SequenceData_CycleBuffer_Test extends Specification {

    Sequence sequence
    SequenceData data

    def setup() {
        sequence = new Sequence(
                circular: true
        )
        data = SequenceData.wrap(sequence, 'ATCAACGCGTTG'.bytes)
    }

    def "test get method"() {
        when:
        def buffer = data.getBuffer(10,4)

        then:
        buffer.get(0) == Nucleotide.T
        buffer.get(1) == Nucleotide.G
        buffer.get(2) == Nucleotide.A
        buffer.get(3) == Nucleotide.T
    }

    def "test get method on revert"() {
        when:
        def buffer = data.getBuffer(10,4).revert()

        then:
        buffer.get(0) == Nucleotide.T
        buffer.get(1) == Nucleotide.A
        buffer.get(2) == Nucleotide.G
        buffer.get(3) == Nucleotide.T
    }

    def "test next"() {
        when:
        def buffer = data.getBuffer(10,4)

        then:
        buffer.next() == Nucleotide.T
        buffer.next() == Nucleotide.G
        buffer.next() == Nucleotide.A
        buffer.next() == Nucleotide.T
        buffer.hasNext() == false
    }

    def "test next on revert"() {
        when:
        def buffer = data.getBuffer(10,4).revert()

        then:
        buffer.next() == Nucleotide.T
        buffer.next() == Nucleotide.A
        buffer.next() == Nucleotide.G
        buffer.next() == Nucleotide.T
        buffer.hasNext() == false
    }

    def "test length"() {
        when:
        def buffer = data.getBuffer(10,4)

        then:
        buffer.length == 4
    }

    def "test length on revert"() {
        when:
        def buffer = data.getBuffer(10,4).revert()

        then:
        buffer.length == 4
    }

    def "test get position after 2 steps"() {
        given:
        def buffer = data.getBuffer(10,4)

        when:
        buffer.next()
        buffer.next()

        then:
        buffer.position == 10 + 2
    }

}
