package cz.mendelu.dnaAnalyser.sequence.stream

import cz.mendelu.dnaAnalyser.batch.Batch
import cz.mendelu.dnaAnalyser.batch.BatchService
import cz.mendelu.dnaAnalyser.sequence.Sequence
import spock.lang.Specification

class BufferedWindowStreamServiceTest extends Specification {

    BufferedWindowStreamService byteBufferStreamService
    BatchService batchService

    File tmpDir
    File tmpFile

    def setup() {
        byteBufferStreamService = new BufferedWindowStreamService()
        byteBufferStreamService.batchService = batchService = Mock(BatchService)

        tmpDir = File.createTempDir("ByteBufferStreamServiceTest-", "")
        byteBufferStreamService.storageDir = tmpDir
        tmpFile = new File(tmpDir, UUID.randomUUID() as String)

        tmpDir.deleteOnExit()
    }

    def "setStorageDir(String) when directory exists"() {
        when:
        byteBufferStreamService.setStorageDir(tmpDir.getAbsolutePath())

        then:
        tmpDir.exists()
    }

    def "setStorageDir(String) when directory don't exists"() {
        setup:
        tmpDir.deleteDir()

        when:
        byteBufferStreamService.setStorageDir(tmpDir.getAbsolutePath())

        then:
        tmpDir.exists()
    }

    def "open(Sequence, int): sequence don't have batch"() {
        given:
        1 * batchService.findBatchFor(_) >> Optional.empty()
        def seq = new Sequence(id: UUID.randomUUID(), bufferId: UUID.fromString(tmpFile.getName()))

        when:
        def result = byteBufferStreamService.open(seq, 4)

        then:
        result.isPresent() == false
    }

    def "open(Sequence = ACTGACTG, 10): try create windows bigger then sequence throw exception."() {
        given:
        tmpFile.write("ACTGACTG")
        1 * batchService.findBatchFor(_) >> Optional.of(new Batch(status: Batch.Status.FINISH))
        def seq = new Sequence(id: UUID.randomUUID(), bufferId: UUID.fromString(tmpFile.getName()), length: 8)

        when:
        byteBufferStreamService.open(seq, 10).get()

        then:
        thrown(IllegalArgumentException)
    }
}
